package brs.visitor.test;
import brs.*;
import brs.visitor.*;
import util.*;
import junit.framework.TestCase;


public class MapBRS_Test extends TestCase {
  IVisitor algo = MapBRS.Singleton;
  
  /**
   * Lambda to replace the root of a tree with its square
   */
  ILambda square = new ILambda() {
    public Object apply(Object... x) {
      BiTree host = (BiTree) x[0];
      int d = ((Integer) host.getRootDat());
      host.setRootDat(d*d);
      return host;
    }
  };
  
  
  /**
   * Lambda to add the count of a tree to its root
   */
  ILambda count = new ILambda() {
    public Object apply(Object... x) {
      BiTree host = (BiTree) x[0];
      host.setRootDat((Integer)host.getRootDat() + (Integer)host.execute(CountBRS.Singleton));
      return host;
    }
  };
  
  
  public void test_empty() {
    BiTree bt = new BiTree();
    
    assertEquals("Empty brs, one lambda", "[]", bt.execute(algo, square).toString());
    assertEquals("Empty brs, two lambdas", "[]", bt.execute(algo, square, count).toString());
  }
  
  public void test_nonEmpty() {
     // Note that the following test code assumes that the lambdas 
    // are applied in the order they are given.
    // If your code performs the application in reverse order, then 
    // modify the expected test results accordingly.

    BiTree bt = new BiTree();
    
    bt.insertRoot(5);
    assertEquals("1 element brs, one lambda", "25\n|_ []\n|_ []", bt.execute(algo, square).toString());
    assertEquals("1 element brs, two lambdas", "626\n|_ []\n|_ []", bt.execute(algo, square, count).toString());
    
    bt = new BiTree();
    bt.insertRoot(2);
    
    BiTree left = new BiTree();
    left.insertRoot(3);
    
    bt.setLeftSubTree(left);
    
    assertEquals("2 element brs, one lambda", "4\n|_ 9\n|  |_ []\n|  |_ []\n|_ []", bt.execute(algo, square).toString());
    assertEquals("2 element brs, two lambdas", "18\n|_ 82\n|  |_ []\n|  |_ []\n|_ []", bt.execute(algo, square, count).toString());
    
    bt = new BiTree();
    bt.insertRoot(2);    
    left = new BiTree();
    left.insertRoot(3);
    bt.setLeftSubTree(left);
    BiTree right = new BiTree();
    right.insertRoot(5);
    bt.setRightSubTree(right);
    
    assertEquals("3 element brs, one lambda", "4\n|_ 9\n|  |_ []\n|  |_ []\n|_ 25\n   |_ []\n   |_ []", bt.execute(algo, square).toString());
    assertEquals("3 element brs, two lambdas", "19\n|_ 82\n|  |_ []\n|  |_ []\n|_ 626\n   |_ []\n   |_ []", bt.execute(algo, square, count).toString());
    
    bt = new BiTree();
    bt.insertRoot(2);    
    left = new BiTree();
    left.insertRoot(3);
    bt.setLeftSubTree(left);
    right = new BiTree();
    right.insertRoot(5);
    bt.setRightSubTree(right);
    
    BiTree bt2 = new BiTree();
    bt2.insertRoot(7);
    bt2.setRightSubTree(bt);
    
    assertEquals("4 element brs, one lambda\n", 
                 "49\n|_ []\n|_ 4\n   |_ 9\n   |  |_ []\n   |  |_ []\n   |_ 25\n      |_ []\n      |_ []",
                 bt2.execute(algo, square).toString());
    assertEquals("4 element brs, two lambdas",
                 "2405\n|_ []\n|_ 19\n   |_ 82\n   |  |_ []\n   |  |_ []\n   |_ 626\n      |_ []\n      |_ []",
                 bt2.execute(algo, square, count).toString());
  }
}
