package btree.visitor;
import btree.*;

/**
 * Concrete implementation of IRACEltFactory.
 */
public class RACEltFactory implements IRACEltFactory {
  /**
   * Singleton design pattern
   */
  public static final RACEltFactory Singleton = new RACEltFactory();
  private RACEltFactory() {}
  
  /**
   * Instantiates a new IDataElt given an INEBTree and an index value
   * indicating which data element to use.
   * @param host An INEBTree
   * @param idx A valid index value to one of the host's data elements.
   * @return A new IDataElt instance 
   */
  public IDataElt makeDataElt(final INEBTree host, final int idx) {
    if(idx >= host.getData().length || idx <0) throw new IllegalArgumentException("idx out of bounds: "+idx);
    return new IDataElt() {
      public INEBTree getBTree() {
        return host;
      }
      
      public int getIdx() {
        return idx;
      }
      
      public Object execute(IRACEltAlgo algo, Object... inp) {
        return algo.dataCase(this, inp);
      }
    };
  }
  
  /**
   * Instantiates a new IBTreeElt given an IBTree
   * @param host An IBTree
   * @return A new IBTreeElt instance 
   */  
  public IBTreeElt makeBTreeElt(final IBTree host) {
    return new IBTreeElt() {
      public IBTree getBTree() {
        return host;
      }
      
      public Object execute(IRACEltAlgo algo, Object... inp) {
        return algo.treeCase(this, inp);
      }
    };
  }
}