package btree.visitor.test;
import btree.*;
import btree.visitor.*;

import junit.framework.TestCase;

public class BTree_Test extends TestCase {
  
  IBTreeFac fac = BTreeFac.Singleton;
  
  public void test_CountElements() {
    IBTreeAlgo algo = CountElements.Singleton;
    
    IBTree mt = fac.makeMTBTree();
    
    assertEquals("Empty tree", 0, mt.execute(algo));
    
    String[] data1 = {"a", "b", "c"};
    IBTree[] children1 = {mt, mt, mt, mt};
    IBTree tree1 = fac.makeNEBTree(data1, children1);

    String[] data2 = {"a"};
    IBTree[] children2 = {mt, mt};
    IBTree tree2 = fac.makeNEBTree(data2, children2);
    
    assertEquals("1 element, no children", 1, tree2.execute(algo));
    
    String[] data3 = {"d", "e"};
    IBTree[] children3 = {tree1, mt, tree2};
    IBTree tree3 = fac.makeNEBTree(data3, children3);
    assertEquals("6 elements, # children = {3, 0, 1}", 6, tree3.execute(algo));

    
  }
  
  public void test_getInsertIndex() {
    IBTree bt1 = fac.makeMTBTree();
    InsertSearchTree algo = new InsertSearchTree(3, fac);
    Object[] data = {5};
    assertEquals("2 -> {5}", 0, algo.getInsertIndex(data, 2));
    assertEquals("7 -> {5}", 1, algo.getInsertIndex(data, 7));
    
    data = new Object[]{5, 10};
    assertEquals("2 -> {5, 10}", 0, algo.getInsertIndex(data, 2));
    assertEquals("7 -> {5, 10}", 1, algo.getInsertIndex(data, 7));
    assertEquals("12 -> {5, 10}", 2, algo.getInsertIndex(data, 12));
  }
  
  public void test_InsertSearchTree() {
    IBTree bt1 = fac.makeMTBTree();
    IBTreeAlgo algo = new InsertSearchTree(3, fac);
    System.out.println("bt1 = \n"+bt1);
    
    
    IBTree bt2 = (IBTree) bt1.execute(algo, 5);
    System.out.println("bt2 = \n"+bt2);
    
    assertEquals("1 element", "[5]\n |_[]\n |_[]\n", bt2.toString()); 
    
    bt2 = (IBTree) bt2.execute(algo, 3);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("2 elements", "[3, 5]\n |_[]\n |_[]\n |_[]\n", bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 8);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("3 elements", "[3, 5, 8]\n |_[]\n |_[]\n |_[]\n |_[]\n", bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 4);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("4 elements", 
                 "[3, 5, 8]\n |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[]\n |_[]\n",
                 bt2.toString()); 
  
    bt2 = (IBTree) bt2.execute(algo, 1);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("5 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[]\n |_[]\n",                 
                 bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 10);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("6 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[]\n |_[10]\n    |_[]\n    |_[]\n",
                 bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 7);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("7 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[7]\n |  |_[]\n |  |_[]\n |_[10]\n    |_[]\n    |_[]\n",
                 bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 6);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("8 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[6, 7]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[10]\n    |_[]\n    |_[]\n",
                 bt2.toString()); 
    
    bt2 = (IBTree) bt2.execute(algo, 15);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("9 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[6, 7]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[10, 15]\n    |_[]\n    |_[]\n    |_[]\n",
                 bt2.toString()); 
    
    bt2 = (IBTree) bt2.execute(algo, 12);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("10 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[6, 7]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[10, 12, 15]\n    |_[]\n    |_[]\n    |_[]\n    |_[]\n",                 
                 bt2.toString()); 
    
    bt2 = (IBTree) bt2.execute(algo, 20);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("11 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[6, 7]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[10, 12, 15]\n    |_[]\n    |_[]\n    |_[]\n    |_[20]\n       |_[]\n       |_[]\n",
                 bt2.toString()); 

    bt2 = (IBTree) bt2.execute(algo, 11);
    System.out.println("bt2 = \n"+bt2);
    assertEquals("12 elements", 
                 "[3, 5, 8]\n |_[1]\n |  |_[]\n |  |_[]\n |_[4]\n |  |_[]\n |  |_[]\n |_[6, 7]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[10, 12, 15]\n    |_[]\n    |_[11]\n    |  |_[]\n    |  |_[]\n    |_[]\n    |_[20]\n       |_[]\n       |_[]\n",
                 bt2.toString()); 
    
    // The following case is not reachable by construction through the 
    // above insertion process 
    IBTree mt = fac.makeMTBTree();
    IBTree bt3 = fac.makeNEBTree(new Object[]{5, 10}, new IBTree[]{mt, mt, mt});
    IBTree bt4 = fac.makeNEBTree(new Object[]{ -2, 15}, new IBTree[]{mt, bt3, mt});
    System.out.println(""+bt4);
    bt4 = (IBTree) bt4.execute(algo, 7);
    System.out.println("bt4 = \n"+bt4);
    assertEquals("sub-order with non-empty child", 
                 "[-2, 15]\n |_[]\n |_[5, 7, 10]\n |  |_[]\n |  |_[]\n |  |_[]\n |  |_[]\n |_[]\n",
                 bt4.toString()); 
    
  }
  
}
