package btree.visitor.test;
import btree.*;
import btree.visitor.*;
import rac.*;
import util.*;
import junit.framework.TestCase;

/**
 * Test class for RACTravers
 */
public class RACTraverse_Test extends TestCase {
  /**
   * common result string used in RACTraverse by the following lambda function. 
   * After RACTraverse is done, result will hold all the data elements in the order 
   * in which they were processed, showing the traversal path through the IBTree.
   */
  String result = ""; 
  
  /**
   * ILambda that is applied to every data element in an IBTree by RACTraverse in the test code.
   * Processes a data element by appending its String representation to the result field above.
   * @param x x[0] is a data element to be processed.   
   * @return null always.
   */
  ILambda lambda = new ILambda() {
    public Object apply(Object...x) {
      result += x[0]+" ";
      return null;
    }
  };
  
  /**
   * Test RACTraverse using a stack as its RAC
   */
  public void test_stack() {
    IRACFactory racFac = new LRSStackFactory();
    IBTreeFac btFac = BTreeFac.Singleton;
    IBTreeAlgo algo = RACTraverse.Singleton;
    IBTreeAlgo insert = new InsertSearchTree(3, btFac);
    
    IBTree bt = btFac.makeMTBTree();
    
    boolean actual = (Boolean) bt.execute(algo);
    boolean expected = false;
    
    assertEquals("empty tree", expected, actual);
    
    bt = (IBTree) bt.execute(insert, 5);
    
    IRAContainer rac = racFac.makeRAC();
    
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("1-element tree", expected, actual);
    assertEquals("1-element tree", "5 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 10);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("2-element tree", expected, actual);
    assertEquals("2-element tree", "5 10 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 15);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("3-element tree", expected, actual);
    assertEquals("3-element tree", "5 10 15 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 2);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("4-element tree", expected, actual);
    assertEquals("4-element tree", "5 10 15 2 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 7);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("5-element tree", expected, actual);
    assertEquals("5-element tree", "5 10 15 7 2 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 12);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("6-element tree", expected, actual);
    assertEquals("6-element tree", "5 10 15 12 7 2 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 20);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("7-element tree", expected, actual);
    assertEquals("7-element tree", "5 10 15 20 12 7 2 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 1);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("8-element tree", expected, actual);
    assertEquals("8-element tree", "5 10 15 20 12 7 1 2 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 3);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("9-element tree", expected, actual);
    assertEquals("9-element tree", "5 10 15 20 12 7 1 2 3 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 6);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("10-element tree", expected, actual);
    assertEquals("10-element tree", "5 10 15 20 12 6 7 1 2 3 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 9);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("11-element tree", expected, actual);
    assertEquals("11-element tree", "5 10 15 20 12 6 7 9 1 2 3 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 8);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("12-element tree", expected, actual);
    assertEquals("12-element tree", "5 10 15 20 12 6 7 9 8 1 2 3 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 13);
    bt = (IBTree) bt.execute(insert, 14);
    bt = (IBTree) bt.execute(insert, 21);
    bt = (IBTree) bt.execute(insert, 22);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("16-element tree", expected, actual);
    assertEquals("16-element tree", "5 10 15 20 21 22 12 13 14 6 7 9 8 1 2 3 ", result);
  }
  
  /**
   * Test RACTraverse using a queue as its RAC
   */
  public void test_queue() {
    IRACFactory racFac = new LRSQueueFactory();
    IBTreeFac btFac = BTreeFac.Singleton;
    IBTreeAlgo algo = RACTraverse.Singleton;
    IBTreeAlgo insert = new InsertSearchTree(3, btFac);
    
    IBTree bt = btFac.makeMTBTree();
    
    boolean actual = (Boolean) bt.execute(algo);
    boolean expected = false;
    
    assertEquals("empty tree", expected, actual);
    
    bt = (IBTree) bt.execute(insert, 5);
    
    IRAContainer rac = racFac.makeRAC();
    
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("1-element tree", expected, actual);
    assertEquals("1-element tree", "5 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 10);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("2-element tree", expected, actual);
    assertEquals("2-element tree", "5 10 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 15);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    assertEquals("3-element tree", expected, actual);
    assertEquals("3-element tree", "5 10 15 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 2);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("4-element tree", expected, actual);
    assertEquals("4-element tree", "5 2 10 15 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 7);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("5-element tree", expected, actual);
    assertEquals("5-element tree", "5 2 10 7 15 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 12);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("6-element tree", expected, actual);
    assertEquals("6-element tree", "5 2 10 7 15 12 ", result);
    
    result = "";
    bt = (IBTree) bt.execute(insert, 20);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("7-element tree", expected, actual);
    assertEquals("7-element tree", "5 2 10 7 15 12 20 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 1);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("8-element tree", expected, actual);
    assertEquals("8-element tree", "5 1 10 2 7 15 12 20 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 3);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("9-element tree", expected, actual);
    assertEquals("9-element tree", "5 1 10 2 7 15 3 12 20 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 6);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("10-element tree", expected, actual);
    assertEquals("10-element tree", "5 1 10 2 6 15 3 7 12 20 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 9);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("11-element tree", expected, actual);
    assertEquals("11-element tree", "5 1 10 2 6 15 3 7 12 9 20 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 8);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("12-element tree", expected, actual);
    assertEquals("12-element tree", "5 1 10 2 6 15 3 7 12 9 20 8 ", result);

    result = "";
    bt = (IBTree) bt.execute(insert, 13);
    bt = (IBTree) bt.execute(insert, 14);
    bt = (IBTree) bt.execute(insert, 21);
    bt = (IBTree) bt.execute(insert, 22);
    actual = (Boolean) bt.execute(algo, rac, lambda);
    expected = true;
    System.out.println("bt = \n"+bt);
    assertEquals("16-element tree", expected, actual);
    assertEquals("16-element tree", "5 1 10 2 6 15 3 7 12 9 13 20 8 14 21 22 ", result);
  }
}
