package life.control;

import life.model.*;
import life.view.*;
import java.awt.Color;

/**
 * Controller class for the Game of Life application
 * Instantiates the model and the view and the adapters that connect them.
 * 
 */
public class LifeApp {
  
  private LifeGUI view;
  private LifeModel model;
  
  private LifeApp() {
    view = new LifeGUI();
    model = new LifeModel(new IViewAdapter(){
      public void update() {
        view.update();
      }
    });
    
    view.setModelAdapter(new IModelAdapter() {
      public int getNRows() {
        return model.getNRows();
      }
      
      public int getNCols() {
        return model.getNCols();
      }
      
      public void start() {
        model.start();
      }
      public void stop() {
        model.stop();
      }
      
      public void reset() {
        model.reset();
      }
      
      public Color getColorAt(int row, int col) {
        return model.getColorAt(row, col);
      }
      
      public String getStrategyName() {
        return model.getStrategyName();
      }
      public String setStrategyByName(String s) {
        return model.setStrategyByName(s);
      }
      
      public Color toNextState(int row, int col) {
        return model.toNextState(row, col);
      }

    });
  }
  
  private void init() {
    view.init();
    view.setVisible(true);
  }
  
  public static void main(String[] args) {
    (new LifeApp()).init();
  }
}