package life.model;

import java.awt.Color;

/**
 * Concrete implementation of ICell
 */
class Cell implements ICell {
  private LifeModel env;
  
  Cell(LifeModel env) {
    this.env = env;
  }

  /**
   * Abstract state of the cell
   */
  private static abstract class AState {
    abstract Color getColor();
    abstract void toNextState();
    abstract Object execute(ICellAlgo algo, Object...params);
  }
  
  /**
   * Dead state of the cell
   */
  private AState deadState = new AState() {
    Color getColor() {
      return Color.WHITE;
    }
    void toNextState() {
      state = live1State;
    }
    Object execute(ICellAlgo algo, Object...params){
      return algo.deadCase(Cell.this, params);
    }
  };
  
  /**
   * First possible live state of the cell
   */
  private AState live1State = new AState() {
    Color getColor() {
      return Color.BLUE;
    }
    void toNextState() {
      //state = deadState;
      state = live2State;
    }
    
    Object execute(ICellAlgo algo, Object...params){
      return algo.live1Case(Cell.this, params);
    }
  };

  /**
   * Second possible live state of the cell
   */
  private AState live2State = new AState() {
    Color getColor() {
      return Color.RED;
    }
    void toNextState() {
      state = deadState;
    }
    
    Object execute(ICellAlgo algo, Object...params){
      return algo.live2Case(Cell.this, params);
    }
  };

  /**
   * Current state of the cell
   */
  private AState state = deadState;
  
  public Color getColor() {
    return state.getColor();
  }
  
  public LifeModel getEnv() {
    return env;
  }
  
  public void setTo(CELLSTATES aState) {
    switch(aState) {
      case DEAD:
        state = deadState;
        break;
      case LIVE1:
        state = live1State;
        break;
      case LIVE2:
        state = live2State;
        break;
      default:
        throw new IllegalStateException("No such state: "+aState);
    }
  }
  public Color toNextState() {
    state.toNextState();
    return state.getColor();
  }

  public Object execute(ICellAlgo algo, Object...params) {
    return state.execute(algo, params);
  }
}
