package life.model;
import util.*;

/**
 * Utility factory class that creates an no-parameter ILambda 
 * that will change the state of the given host cell to the 
 * given state when the lambda is applied.
 */
public class CellStateChangeFactory {
  public static final CellStateChangeFactory Singleton = new CellStateChangeFactory();
  private CellStateChangeFactory() {}
  
  /**
   * Returns an ILambda that will change the given cell to the 
   * given state when it is applied.  That is if 
   * lambda = factory.makeLambda(aCell, aState)
   * then lambda.apply() will change the state of aCell to aState.
   * @param host The ICell whose state is to be changed.
   * @param state The state to change to, as defined by the enum, ICell.CELLSTATES
   * @return An ILambda whose apply takes no parameters and returns null.
   */
  public ILambda makeLambda(final ICell host, final ICell.CELLSTATES state) {
    return new ILambda() {
      public Object apply(Object... nu) {
        host.setTo(state);
        return null;
      }
    };
  }
}