package life.model;

import util.*;
import lrs.*;


/**
 * Evolution strategy that only affects dead and live1 states. 
 * Recreates the classic live/die/reborn rules.
 * @param params  params[0] = row, params[1] = col, params[2] = LRStruct to hold state change lambdas for later execution.
 */
public class Evolve1Algo extends AEvolveAlgo { //ACellAlgo {
  public static final  Evolve1Algo Singleton = new Evolve1Algo();
  private Evolve1Algo() {
    /**
     * Adds one to the number of dead and live1 cells
     * depending on the state of the host cell.
     * ((int[])params[0])[0] is the current count of dead cells
     * ((int[])params[0])[1] is the current count of live1 cells
     * @param params  params[0] is an int[] with at least two elements containing the count of dead and live1 cells..
     * @return null, always.
     */
    countAlgo = new ACellAlgo() {
      public Object deadCase(ICell host, Object... params) {
        ((int[]) params[0])[0]++;
        return null;
      }
      
      public Object live1Case(ICell host, Object... params) {
        ((int[]) params[0])[1]++;
        return null;
      }
    };  
  }
  
  private CellStateChangeFactory fac = CellStateChangeFactory.Singleton;
  
  public Object deadCase(final ICell host, final Object... params) {
    final int[] count = countNeighbors(host.getEnv(), (Integer)params[0], (Integer)params[1], 1);
    if(5==count[0]) { // reborn if exactly 3 live nieghbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.LIVE1)); 
    }
    return null;
  }
  
  
  public Object live1Case(final ICell host, final Object... params) {
    final int[] count = countNeighbors(host.getEnv(), (Integer)params[0], (Integer)params[1], 1);
    if((count[1]<2) || (count[1] >3)) { // Dies if less than 2 ot more than 4 live neighbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.DEAD)); 
    }
    return null;
    
  }

}