package life.model;

import util.*;
import lrs.*;


/**
 * Evolution algorithm that works with dead, live1 and live2 states. 
 * Alone, the two states follow the classic live/die/reborn rules.
 * 2 or more live2 neighbors will kill a live1 cell though.
 * 
 * param params  params[0] = row, params[1] = col, params[2] = LRStruct to hold state change lambdas for later execution.
 */
public class Evolve2Algo extends AEvolveAlgo {
  public static final  Evolve2Algo Singleton = new Evolve2Algo();
  private Evolve2Algo() {
    
    // Algo used to count the neighbors.
    countAlgo = new ACellAlgo() {
          public Object deadCase(ICell host, Object... params) {
            ((int[])params[0])[0]++;
            return null;
          }
          public Object live1Case(ICell host, Object... params) {
            ((int[])params[0])[1]++;
            return null;
          }
          public Object live2Case(ICell host, Object... params) {
            ((int[])params[0])[2]++;
            return null;
          }        
        };
  }

  private CellStateChangeFactory fac = CellStateChangeFactory.Singleton;

  public Object deadCase(final ICell host, final Object... params) {
    final int[] count = countNeighbors(host.getEnv(), (Integer)params[0], (Integer)params[1], 1);
    if(3==count[1] && 3!= count[2]) { // reborn if exactly 3 live nieghbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.LIVE1)); 
    }
    else if(3==count[2] && 3!= count[1]) { // reborn if exactly 3 live nieghbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.LIVE2)); 
    }
    else if(3==count[2] && 3== count[1]){
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,
                                                       (0.5 < Math.random()) ?
                                                         ICell.CELLSTATES.LIVE1:
                                                         ICell.CELLSTATES.LIVE2)); 
    }
    return null;
  }
  
  
  public Object live1Case(final ICell host, final Object... params) {
    final int[] count = countNeighbors(host.getEnv(), (Integer)params[0], (Integer)params[1], 1);
    if(count[2] >1) { // 
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.LIVE2)); 
    }
    else if(count[1]<2 || count[1] >3 ) { // Dies if less than 2 or more than 4 live neighbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.DEAD)); 
    }
    return null;    
  }
  
  public Object live2Case(final ICell host, final Object... params) {
    final int[] count = countNeighbors(host.getEnv(), (Integer)params[0], (Integer)params[1], 1);
    if((count[2]<2) || (count[2] >3)) { // Dies if less than 2 ot more than 4 live neighbors
      ((LRStruct)params[2]).insertFront(fac.makeLambda(host,ICell.CELLSTATES.DEAD)); 
    }
    return null;    
  }  
}