package life.model;
import java.awt.Color;

/**
 * Interface that represents an abstract cell
 */
public interface ICell {
  /**
   * Enumeration that lists out the possible cell states
   */
  public static enum CELLSTATES {DEAD, LIVE1, LIVE2}

  /**
   * Get the current color of this cell
   */
  public Color getColor();
  
  /**
   * Get the LifeModel environment of this cell.
   */
  public LifeModel getEnv();
  
  /**
   * Set the cell to the state specified by a value from the above enumeration.
   */
  public void setTo(CELLSTATES aState);
  
  /**
   * Change the state to the next possible state of the cell
   * Returns the current color, after changing states.
   */
  public Color toNextState();
  
  /**
   * Executes a visitor algorithm on this cell
   */
  public Object execute(ICellAlgo algo, Object...params);
}