package life.view;
import java.awt.Color;

/**
 * The adapter from the view to the model
 */
public interface IModelAdapter {
  /**
   * Get the number of rows of cells in the system.
   */
  public int getNRows();
  /**
   * Get the number of columns of cells in the system.
   */
  public int getNCols();
  
  /**
   * Start the model's periodic updating of the cells
   */
  public void start();
  
  /**
   * STop the model's periodic updating of the cells
   */
  public void stop();
  /**
   * Reset the cells to all dead.
   */
  public void reset();
  
  /**
   * GEt the color of the cell at row, col.
   */
  public Color getColorAt(int row, int col);
  
  /**
   * Get the name of the current strategy in use
   */
  public String getStrategyName();
  
  /**
   * SEt the current strategy to that specified by the given fully qualified class name.
   */
  public String setStrategyByName(String s);
  
  /**
   * Set the cell at row, col to the its next possible state.
   */
  public Color toNextState(int row, int col);
  

}