package life.view;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;



/**
 * View for the system
 */
public class LifeGUI extends AFrame {
  private static final long serialVersionUID = 200804191421L;
  
  private IModelAdapter model;
  
  public LifeGUI(){
    super("Game of Life");
  }
  
  public void setModelAdapter(IModelAdapter model) {
    this.model = model;
  }
  
  private JButton[][] cells;
  
  /**
   * Initializes the view
   * Must be called AFTER the model adapter is connected 
   * and the model is instantiated
   */
  public void init() {
    strategyTF.setText(model.getStrategyName());
    cells = new JButton[model.getNRows()][model.getNCols()];
    makeCells();
    update();
    validate();
    pack();
  }
  
  private JPanel controlPnl;
  private JPanel displayPnl;
  private JButton startBtn;
  private JButton stopBtn;
  private JButton resetBtn;
  private JTextField strategyTF;
  private JButton changeBtn;
  
  protected void initialize() {
    Container cp = getContentPane();

    controlPnl = new JPanel();
    displayPnl = new JPanel();
    
    cp.add(controlPnl, BorderLayout.NORTH);
    cp.add(displayPnl, BorderLayout.CENTER);
    
    
    startBtn = new JButton("Start");
    startBtn.setEnabled(true);
    startBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        model.start();
        stopBtn.setEnabled(true);
        startBtn.setEnabled(false);
      }
    });
    
    stopBtn = new JButton("Stop");
    stopBtn.setEnabled(false);
    stopBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        model.stop();
        startBtn.setEnabled(true);
        stopBtn.setEnabled(false);
      }
    });
    
    resetBtn = new JButton("Reset");
    resetBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        model.reset();
      }
    });
    
    strategyTF = new JTextField();
    
    changeBtn = new JButton("Change Strategy");
    changeBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        strategyTF.setText(model.setStrategyByName(strategyTF.getText()));
      }
    });
    
    controlPnl.add(startBtn);
    controlPnl.add(stopBtn);
    controlPnl.add(resetBtn);  
    controlPnl.add(strategyTF);
    controlPnl.add(changeBtn);
  }
  
  public void update() {
    for(int i = 0; i< model.getNRows(); i++) {
      for(int j=0; j< model.getNCols(); j++){
        cells[i][j].setBackground(model.getColorAt(i,j));
      }
    }
    repaint();
  }
  
  private void makeCells(){
    displayPnl.setLayout(new GridLayout(cells.length, cells[0].length));
    for(int i = 0; i< model.getNRows(); i++) {
      for(int j=0; j< model.getNCols(); j++){
        final int row = i;
        final int col = j;
        cells[i][j] = new JButton() {
          private static final long serialVersionUID = 200804191447L;
          private int colorIdx = 0;           
          {
            //setBackground(colors[colorIdx++]);
            addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                Color c = LifeGUI.this.model.toNextState(row, col);
                setBackground(c);
                repaint();
              }
            });
          }
        };
        
        displayPnl.add(cells[i][j]);
      }
    }
  }
}