package lrs.visitor;
import lrs.*;
import java.util.*;

/**
 * Returns a LRStruct whose first is the minimum value 
 * in the host list
 */
public class FindMin implements IAlgo {

  private Comparator _order;

  public FindMin(Comparator ord) {
      _order = ord;
  }

  /**
   * Helper algo that accumulates the location of the minimum 
   * value in the list and then removes it.
   * @param min min[0] is the LRStruct which holds the minimum value so far.
   * @return The minimum value.
   */
  private final IAlgo helper = new IAlgo() {
    public Object emptyCase(LRStruct host, Object...min) {
      return min[0];
    }
    
    public Object nonEmptyCase(LRStruct host, Object...min) {
      return host.getRest().execute(this,
                                    (_order.compare(((LRStruct)min[0]).getFirst(), host.getFirst()) < 0)?
                                      min[0] : host);
    }
  };


  /**
  * Throws an IllegalState exception because there is nothing to
  * remove from an empty list.
  */
  public Object emptyCase(LRStruct host, Object... nu) {
    throw new IllegalStateException("LRStruct is empty, nothing to remove!");
  }



  /**
  * Based on the comparison between first and n,
  * removes the front or recurs!
  * @param host a non-empty LRStruct.
  * @param n an Object to be inserted in order into host,
  *   based on the given Comparator.
  * @return LRStruct
  */
  public Object nonEmptyCase(LRStruct host, Object... nu) {
    return host.getRest().execute(helper, host);
  }

}



