package schedule.test;
import schedule.*;
import util.*;
import junit.framework.TestCase;


public class TaskScheduler_Test extends TestCase {
  
  private ILambda noOp = makeTask("No-op");
  private int maxTime = 1; // 1 second
  
  public void test_TimePriorityComp() {
    TaskScheduler sched = new TaskScheduler(new TimePriorityComp(maxTime), noOp);
    
    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 1);
    
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 3);
    
    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(1100) ; // 1.1 sec wait
    sched.addTask(makeTask("task 2"), 1);

    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());
    
    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 1);
   
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 2);
    sleep(1200) ; // 1.2 s wait
    sched.addTask(makeTask("task 3"), 1);
    sched.addTask(makeTask("task 4"), 5);
    
   
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "task 3", sched.getNextTask().apply());
    assertEquals("Run fourth task", "task 4", sched.getNextTask().apply());
    assertEquals("Run fifth task", "No-op", sched.getNextTask().apply());
  
  }
  
  public void test_TimeComp() {
    TaskScheduler sched = new TaskScheduler(new TimeComp(), noOp);
    
    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 1);
    
    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 3);
    
    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(1100) ; // 1.1 sec wait
    sched.addTask(makeTask("task 2"), 1);

    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());
    
    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 1);
   
    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 2);
    sleep(1200) ; // 1.2 s wait
    sched.addTask(makeTask("task 3"), 1);
    sched.addTask(makeTask("task 4"), 5);
    
   
    assertEquals("Run first task", "task 1", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "task 3", sched.getNextTask().apply());
    assertEquals("Run fourth task", "task 4", sched.getNextTask().apply());
    assertEquals("Run fifth task", "No-op", sched.getNextTask().apply());
  
  }

  public void test_PriorityComp() {
    TaskScheduler sched = new TaskScheduler(new PriorityComp(), noOp);
    
    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 1);
    
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sched.addTask(makeTask("task 2"), 3);
    
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(1100) ; // 1.1 sec wait
    sched.addTask(makeTask("task 2"), 1);

    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());
    
    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 1);
   
    assertEquals("Run first task", "task 2", sched.getNextTask().apply());
    assertEquals("Run second task", "task 1", sched.getNextTask().apply());
    assertEquals("Run third task", "No-op", sched.getNextTask().apply());

    sched.addTask(makeTask("task 1"), 3);
    sleep(800) ; // .8 s wait
    sched.addTask(makeTask("task 2"), 2);
    sleep(1200) ; // 1.2 s wait
    sched.addTask(makeTask("task 3"), 1);
    sched.addTask(makeTask("task 4"), 5);
    
   
    assertEquals("Run first task", "task 3", sched.getNextTask().apply());
    assertEquals("Run second task", "task 2", sched.getNextTask().apply());
    assertEquals("Run third task", "task 1", sched.getNextTask().apply());
    assertEquals("Run fourth task", "task 4", sched.getNextTask().apply());
    assertEquals("Run fifth task", "No-op", sched.getNextTask().apply());
  
  }
  
  /** 
   * Utility method to make an ILambda whose apply() method
   * returns the given String
   */
  private ILambda makeTask(final String s) {
    return new ILambda() {
      public Object apply(Object...nu) {
        return s;
      }
    };
  }
  
  /**
   * Utility method that will cause the current operation to suspend (wait)
   * for the given number of milliseconds
   */
  private void sleep(long ms){
    try{
      Thread.sleep(ms);  // This current thread of operation will sleep for ms milliseconds.
    }
    catch(Exception e) {
      System.err.println("Thread sleep exception: "+e);
    }
  }
}
